/***************************************************************************
 *   Copyright (C) 2007 by Jackie Team   *
 *   jackie.team@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "components.h"
#include <id3v2tag.h>
#include <mpegfile.h>
#include <fileref.h>
#include <tag.h>
#include <attachedpictureframe.h>
#include <vorbisfile.h>
//#include <wx/base64.h>
#include <flacfile.h>
const TagLib::String  Base64Handler::base64_chars=
             "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
             "abcdefghijklmnopqrstuvwxyz"
             "0123456789+/";

BEGIN_EVENT_TABLE (AdvPropertiesDialog , wxFrame )
    EVT_BUTTON(ID_OK,AdvPropertiesDialog::onOk)
END_EVENT_TABLE()


AdvPropertiesDialog::AdvPropertiesDialog(MonChoicebook* choice,wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style ) : wxFrame( parent, id, title, pos, size, style ){
    mc=choice;
    MaListe* currentMl=mc->getCurrentList();
    long index = currentMl->getSelection(-1);
    wxString file=currentMl->getRowText(index,MaListe::DIR_COL)+wxFileName::GetPathSeparator()+currentMl->getRowText(index,MaListe::NAME_COL);
    wxFileName fn(file);
    wxBoxSizer *bs=new wxBoxSizer(wxVERTICAL);
    wxPanel *pInfo=new wxPanel(this);

    //Mime related infos
    wxFileType *ft=wxTheMimeTypesManager->GetFileTypeFromExtension(fn.GetExt());
    wxIconLocation iconLoc;
    wxLogNull();
    wxBitmap typeBitmap;
    wxString typeDescr;
    if(ft!=NULL){
        ft->GetIcon(&iconLoc);
        wxIcon typeIcon(iconLoc);
        typeBitmap.CopyFromIcon(typeIcon);
        ft->GetDescription(&typeDescr);
    }

    wxBitmapButton *btnIco = new wxBitmapButton(this,wxID_ANY,typeBitmap),*btnID3=NULL;

    //file system related infos
    wxString sizeF = fn.GetHumanReadableSize();
    wxDateTime accesT,modT,createT;
    fn.GetTimes(&accesT,&modT,&createT);
    wxString create=wxDateTimeToStr(createT);
    wxString modif=wxDateTimeToStr(modT);
    wxString access=wxDateTimeToStr(accesT);


    wxString bpm,comm,compo,enco,copy,lyr;
    wxBitmap pictFile;
    //tag related infos
    if (mc->isID3V2()){
        #ifdef __WXMSW__ //strange encoding bug
        TagLib::MPEG::File fic((const char*)file.mb_str(wxConvUTF8),true,TagLib::AudioProperties::Average);
        #else
        TagLib::MPEG::File fic((const char*)file.fn_str(),true,TagLib::AudioProperties::Average);
        #endif

        //read ID3V2 tags of the file
        if(fic.ID3v2Tag()!=NULL){//ID3V2tag
            //http://www.id3.org/id3v2.4.0-frames
             TagLib::ID3v2::FrameList lbpm = fic.ID3v2Tag()->frameListMap()["TBPM"];//Beat per minute
            if(lbpm.size()!=0){
                TagLib::String bpm2 = lbpm.front()->toString();
                            bpm = wxString(bpm2.toCString(true),wxConvUTF8);
            }
            TagLib::ID3v2::FrameList lpict = fic.ID3v2Tag()->frameListMap()["APIC"];//picture
            if(lpict.size()!=0){
                TagLib::ID3v2::AttachedPictureFrame pict(lpict.front()->render());
                wxString type(pict.mimeType().toCString(true),wxConvUTF8);
                //First write the picture into a file
                wxString name=wxFileName::CreateTempFileName(_T("moadvpict"));
                wxFile *f;
                f = new wxFile(name.c_str(),wxFile::write);
                f->Write((const void*)pict.picture().data(),pict.picture().size());
                f->Close();

                //Next use the file to make a wxBitmap
                wxImage img;
                if(type.MakeLower().Find(_T("jpeg"))){
                    img = wxImage(name,wxBITMAP_TYPE_JPEG);
                }else if(type.MakeLower().Find(_T("png"))){
                    img = wxImage(name,wxBITMAP_TYPE_PNG);
                }else if(type.MakeLower().Find(_T("gif"))){
                    img = wxImage(name,wxBITMAP_TYPE_GIF);
                }
                pictFile = wxBitmap(img);
                //Delete the file
                wxRemoveFile(name);
                btnID3  = new wxBitmapButton(this,wxID_ANY,pictFile);

            }
            TagLib::ID3v2::FrameList lcomm = fic.ID3v2Tag()->frameListMap()["COMM"];//comment
            if(lcomm.size()!=0){
                TagLib::String comment = lcomm.front()->toString();
                comm = wxString(comment.toCString(true),wxConvUTF8);
            }
            TagLib::ID3v2::FrameList lcomposer = fic.ID3v2Tag()->frameListMap()["TCOM"];//composer
            if(lcomposer.size()!=0){
                TagLib::String composer = lcomposer.front()->toString();
                compo= wxString(composer.toCString(true),wxConvUTF8);
            }
            TagLib::ID3v2::FrameList lenc = fic.ID3v2Tag()->frameListMap()["TENC"];//encoder
            if(lenc.size()!=0){
                TagLib::String encoder = lenc.front()->toString();
                enco= wxString(encoder.toCString(true),wxConvUTF8);
                wxMessageBox(enco);
            }
            TagLib::ID3v2::FrameList lcopy = fic.ID3v2Tag()->frameListMap()["TCOP"];//copyright
            if(lcopy.size()!=0){
                TagLib::String copyright = lcopy.front()->toString();
                copy= wxString(copyright.toCString(true),wxConvUTF8);
            }
            TagLib::ID3v2::FrameList llyr = fic.ID3v2Tag()->frameListMap()["USLT"];//lyrics
            if(llyr.size()!=0){
                TagLib::String lyrics = llyr.front()->toString();
                lyr = wxString(lyrics.toCString(true),wxConvUTF8);
            }

        }


    }else if(mc->getCurrentList()==mc->getMListeOgg()){//http://reactor-core.org/ogg-tagging.html
        #ifdef __WXMSW__ //strange encoding bug
        TagLib::Ogg::Vorbis::File  fic((const char*)file.mb_str(wxConvUTF8),false,TagLib::AudioProperties::Average);
        #else
        TagLib::Ogg::Vorbis::File  fic((const char*)file.fn_str(),false,TagLib::AudioProperties::Average);
        #endif
        if(fic.tag()!=NULL){
            if(fic.tag()->fieldListMap()["COMMENT"].size()!=0){
               TagLib::String comment = fic.tag()->fieldListMap()["COMMENT"].front();
               comm = wxString(comment.toCString(true),wxConvUTF8);
               wxMessageBox(comm);
            }
            wxString mime;
            if(fic.tag()->fieldListMap()["COVERARTMIME"].size()!=0){//get cover not very official
               TagLib::String mim = fic.tag()->fieldListMap()["COVERARTMIME"].front();
               mime = wxString(mim.toCString(true),wxConvUTF8);
               if(fic.tag()->fieldListMap()["COVERART"].size()!=0){
                   TagLib::String encData = fic.tag()->fieldListMap()["COVERART"].front();//base 64 encoded data
                   //FIXME encData seems not to have the right length, it's a taglib bug, maybe fixed some day
                   TagLib::String decData(Base64Handler::base64_decode(encData.to8Bit()));

                   wxString name=wxFileName::CreateTempFileName(_T("moadvpict"));
                   wxFile *f;
                   f = new wxFile(name.c_str(),wxFile::write);
                   f->Write(decData.toCString(false),fic.tag()->fieldListMap()["COVERART"].front().size());
                   f->Close();
                   //Next use the file to make a wxBitmap
                   wxMessageBox(wxString::Format(_T("%d"),decData.size()));
                   wxImage img;
                   if(mime.MakeLower().Find(_T("jpeg"))){
                       img = wxImage(name,wxBITMAP_TYPE_JPEG);
                   }else if(mime.MakeLower().Find(_T("png"))){
                       img = wxImage(name,wxBITMAP_TYPE_PNG);
                   }else if(mime.MakeLower().Find(_T("gif"))){
                       img = wxImage(name,wxBITMAP_TYPE_GIF);
                   }
                   pictFile = wxBitmap(img);
                   //Delete the file
                   wxRemoveFile(name);
                   //btnID3  = new wxBitmapButton(this,wxID_ANY,pictFile);//commented until the taglib bug is fixed

               }
            }
        }

    }else if(mc->getCurrentList()==mc->getMListeFlac() ){
        #ifdef __WXMSW__ //strange encoding bug
        TagLib::FLAC::File  fic((const char*)file.mb_str(wxConvUTF8),false,TagLib::AudioProperties::Average);
        #else
        TagLib::FLAC::File  fic((const char*)file.fn_str(),false,TagLib::AudioProperties::Average);
        #endif
        //tags= fic.xiphComment();
        if(fic.xiphComment()!=NULL){
            if(fic.xiphComment()->fieldListMap()["COMMENT"].size()!=0){
               TagLib::String comment = fic.xiphComment()->fieldListMap()["COMMENT"].front();
               comm = wxString(comment.toCString(true),wxConvUTF8);
               wxMessageBox(comm);
            }
            //FIXME find how to get the content of the METADATA_PICTURE_BLOCK to pass it to getFlacPicture
            //http://flac.sourceforge.net/format.html#metadata_block_picture not a Xiph Commennt
/*            #ifdef __WXMSW__ //strange encoding bug
            wxFile f(file,wxFile::read);
            #else
            wxFile f(file,wxFile::read);
            #endif
            f.Seek(4);
            //TagLib::ByteVector stream = fic.streamInfoData();
            bool found=false, end=false;
            char type1[2],len1[3];
            int pos=4;
            //while (!found && !end){
              f.Read(type,2);
              f.Read(len,3);
              wxString t=wxString::Format(_T("%d"),type),l=wxString::Format(_T("%d"),len);
              wxMessageBox(t+ wxString(_T("aa")) + l);
              int type = (int) stream.mid(pos,2).toUInt();
              pos+=2;
              int data_len = (int) stream.mid(pos,3).toUInt();
              pos+= 3 +data_len;
              if (type>127)
                 end=true;

           }*/
        }
    }


    if(btnID3!=NULL)
        bs->Add(btnID3,0, wxALL, 5 );
    okButton = new wxButton(this,ID_OK,_("OK"));
    bs->Add(btnIco,0, wxALL, 5 );
    bs->Add(pInfo);
    bs->Add(okButton);
    this->SetSizer(bs);
    bs->Fit(this);
    Show();

}

/* *************************************************************************** */

wxBitmap AdvPropertiesDialog::getFlacPicture(TagLib::ByteVector data)
{
       wxBitmap picture;
	if (data == NULL)
		return picture;

	if (data.size() < 32)
		return picture;

	int pos = 0;
	//int type = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	int mimetype_length = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	TagLib::String mime_type = data.mid(pos,mimetype_length).data();
	pos += mimetype_length;
        wxString mtype(mime_type.toCString(true),wxConvUTF8);

	int description_length = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	TagLib::String description = data.mid(pos,description_length).data();
	pos += description_length;

	//int width = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	//int height = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	//int color_depth = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	//int indexed_colors = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	int data_length = (int) data.mid (pos, 4).toUInt ();
	pos += 4;

	char *picture_data = data.mid (pos, data_length).data();
        wxString name=wxFileName::CreateTempFileName(_T("moadvpict"));
        wxFile *f;
        f = new wxFile(name.c_str(),wxFile::write);
        f->Write((const void*)picture_data,data_length);
        f->Close();

        //Next use the file to make a wxBitmap
        wxImage img;
        if(mtype.MakeLower().Find(_T("jpeg"))){
            img = wxImage(name,wxBITMAP_TYPE_JPEG);
        }else if(mtype.MakeLower().Find(_T("png"))){
            img = wxImage(name,wxBITMAP_TYPE_PNG);
        }else if(mtype.MakeLower().Find(_T("gif"))){
            img = wxImage(name,wxBITMAP_TYPE_GIF);
        }
        picture = wxBitmap(img);
        //Delete the file
        wxRemoveFile(name);

        return picture;
}


/* *************************************************************************** */

wxString AdvPropertiesDialog::wxDateTimeToStr(const wxDateTime& dt){
    return wxString::Format(_("%d/%d/%d at %d:%d:%d"),dt.GetMonth(),dt.GetDay(),dt.GetYear(), dt.GetHour(),dt.GetMinute(),dt.GetSecond());
}

/* *************************************************************************** */

void AdvPropertiesDialog::onOk(wxCommandEvent& e){
    Destroy();
}

/* **************************************************************************
   *************************************************************************** */
std::string Base64Handler::base64_decode(std::string const& encoded_string) {
  //http://www.adp-gmbh.ch/cpp/common/base64.html
  int in_len = encoded_string.size();
  int i = 0;
  int j = 0;
  int in_ = 0;
  char char_array_4[4], char_array_3[3];
  std::string ret;

  while (in_len-- && ( encoded_string[in_] != '=') && is_base64(encoded_string[in_])) {
    char_array_4[i++] = encoded_string[in_]; in_++;
    if (i ==4) {
      for (i = 0; i <4; i++)
        char_array_4[i] = base64_chars.find(TagLib::String((char)char_array_4[i],TagLib::String::Latin1));

      char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
      char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
      char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

      for (i = 0; (i < 3); i++)
        ret += char_array_3[i];
      i = 0;
    }
  }

  if (i) {
    for (j = i; j <4; j++)
      char_array_4[j] = 0;

    for (j = 0; j <4; j++)
      char_array_4[j] = base64_chars.find(TagLib::String((char)char_array_4[j],TagLib::String::Latin1));

    char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
    char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
    char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

    for (j = 0; (j < i - 1); j++) ret += (char)char_array_3[j];
  }

  std::cout << encoded_string.size() << endl;
  return ret;
}

/* *************************************************************************** */

std::string Base64Handler::base64_encode(char const* bytes_to_encode, unsigned int in_len) {
  std::string ret;
  int i = 0;
  int j = 0;
  unsigned char char_array_3[3];
  unsigned char char_array_4[4];

  while (in_len--) {
    char_array_3[i++] = *(bytes_to_encode++);
    if (i == 3) {
      char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
      char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
      char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
      char_array_4[3] = char_array_3[2] & 0x3f;

      for(i = 0; (i <4) ; i++)
        ret += base64_chars[char_array_4[i]];
      i = 0;
    }
  }

  if (i)
  {
    for(j = i; j < 3; j++)
      char_array_3[j] = '\0';

    char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
    char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
    char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
    char_array_4[3] = char_array_3[2] & 0x3f;

    for (j = 0; (j < i + 1); j++)
      ret += base64_chars[char_array_4[j]];

    while((i++ < 3))
      ret += '=';

  }

  return ret;

}
